#include <LiquidCrystal_I2C.h>
#include <Wire.h>
#include <TMRpcm.h>
#include <SPI.h>
#include <SD.h>

// Configuration des pins
const int buttonPin = 2;       // Broche du bouton
const int speakerPin = 9;      // Broche pour le haut-parleur (doit être une broche PWM)
const int SD_ChipSelectPin = 10; // Broche pour la sélection de la carte SD

// Durée et score du jeu
const unsigned long gameDuration = 180000; // 3 minutes en millisecondes
unsigned long gameStartTime;
int score = 0;

// État du jeu
bool isFishing = false;

// LCD I2C
LiquidCrystal_I2C lcd(0x27, 20, 4);  

// Audio
TMRpcm audio;

// Types d'objets
enum ObjectType { FISH, SHOE, SHARK };

void setup() {
  pinMode(buttonPin, INPUT_PULLUP);
  audio.speakerPin = speakerPin;
  
  // Initialisation de la carte SD
  if (!SD.begin(SD_ChipSelectPin)) {
    Serial.println("Échec de l'initialisation de la carte SD");
    while (1); // Boucle infinie pour arrêter le programme
  }

  // Initialisation de l'écran LCD
  lcd.init();
  lcd.backlight();

  // Configuration initiale du jeu
  Serial.begin(9600);
}

void loop() {
  // Gestion du début du jeu
  if (digitalRead(buttonPin) == LOW) {
    if (!isFishing) {
      isFishing = true;
      gameStartTime = millis();
      score = 0;
      lcd.clear();
      lcd.print("Get ready to fish!");
      delay(2000);
      lcd.clear();
    }
  }

  // Logique du jeu
  if (isFishing) {
    if (millis() - gameStartTime < gameDuration) {
      playFishingGame();
    } else {
      // Fin du jeu
      isFishing = false;
      lcd.clear();
      lcd.print("Time's up! Score: ");
      lcd.print(score);
      delay(5000); // Afficher le score final pendant 5 secondes
    }
  }
}

void playFishingGame() {
  // Délai aléatoire pour simuler l'attente d'un poisson
  unsigned long showFishTime = random(1000, 10000);
  delay(showFishTime);

  // Sélectionner aléatoirement un objet
  ObjectType objectType = static_cast<ObjectType>(random(0, 3)); // 0 = FISH, 1 = SHOE, 2 = SHARK

  // Affichage de l'objet et activation du son et du vibreur
  lcd.clear();
  switch (objectType) {
    case FISH:
      lcd.print("Fish!");
      audio.play("fish.wav"); // Jouer le son "fish.wav"
      break;
    case SHOE:
      lcd.print("Shoe!");
      audio.play("shoe.wav"); // Jouer le son "shoe.wav"
      break;
    case SHARK:
      lcd.print("Shark!");
      audio.play("shark.wav"); // Jouer le son "shark.wav"
      break;
  }
  // Durée de présence de l'objet
  unsigned long presenceDuration = (objectType == SHARK) ? 500 : 1000;
  delay(presenceDuration);

  // Logique de pêche
  if (digitalRead(buttonPin) == HIGH) {
    // Si le bouton est relâché (pêcher l'objet)
    switch (objectType) {
      case FISH:
        score += 5; // Augmenter le score pour un poisson
        break;
      case SHOE:
        score -= 3; // Diminuer le score pour une chaussure
        break;
      case SHARK:
        score += 10; // Augmenter beaucoup le score pour un requin
        break;
    }
    lcd.clear();
    lcd.print("Caught! Score: ");
    lcd.print(score);
  } else {
    lcd.clear();
    lcd.print("Missed! Score: ");
    lcd.print(score);
  }

  // Délai avant le prochain objet
  delay(2000);
}
